var axios = require('axios')

/**
 * Scape
 *
 * @param {Object} opts Object of config options
 *   @param {String} [opts.hostname] hostname
 *   @param {String} [opts.hostnameUpload] hostnameUpload
 *   @param {String} [opts.apiToken] apiToken
 *   @param {String} [opts.accessToken] accessToken
 */
var Scape = function (opts = {}) {
    Object.assign(this, {
        hostname:       'http://scape.ideea.co.uk/api/v1',
        hostnameUpload: 'https://scape-upload.ideea.co.uk/scape',
        apiToken: false,
        accessToken: false,
    }, opts)
}

/**
 * .getHeaders()
 *
 * @returns {Array} 
 */
Scape.prototype.getHeaders = function () {
    var headers = {}
    if (this.apiToken) headers['X-Api-Token'] = this.apiToken
    if (this.accessToken) headers['Authorization'] = 'Bearer ' + this.accessToken
    return headers
}


// /////////////////////////////////
// Buckets
// /////////////////////////////////
/**
 * GET /buckets/creates
 *
 * Get buckets
 *
 * @returns {Promise} 
 */
Scape.prototype.createBucket = function (bucket) {
    return new Promise((resolve, reject) => {
        axios.post(`${this.hostname}/buckets/create`, bucket, {
            headers: this.getHeaders(),
        })
        .then(res => resolve(res.data))
        .catch(err => reject(err))
    })
}

/**
 * GET /buckets
 *
 * Get buckets
 *
 * @returns {Promise} 
 */
Scape.prototype.getBuckets = function (opts = {}) {
    var params = {}
    // if (opts.getTotalImages) params.getTotalImages = 'true'

    return new Promise((resolve, reject) => {
        axios.get(`${this.hostname}/buckets`, {
            headers: this.getHeaders(),
            params: params,
        })
        .then(res => resolve(res.data))
        .catch(err => reject(err))
    })
}

/**
 * GET /buckets/{uuid}
 *
 * Get single bucket
 *
 * @returns {Promise} 
 */
Scape.prototype.getBucket = function (UUID, opts = {}) {
    var params = {}
    if (opts.with) params.with = opts.with

    return new Promise((resolve, reject) => {
        axios.get(`${this.hostname}/buckets/${UUID}`, {
            headers: this.getHeaders(),
            params: params,
        })
        .then(res => resolve(res.data))
        .catch(err => reject(err))
    })
}

/**
 * DEL /buckets/{uuid}/delete
 *
 * Delete bucket
 *
 * @returns {Promise} 
 */
Scape.prototype.deleteBucket = function (UUID) {
    return new Promise((resolve, reject) => {
        axios.delete(`${this.hostname}/buckets/${UUID}/delete`, {
            headers: this.getHeaders(),
        })
        .then(res => resolve(res.data))
        .catch(err => reject(err))
    })
}


// /////////////////////////////////
// Images
// /////////////////////////////////
/**
 * GET /images
 *
 * Get images
 *
 * @returns {Promise} 
 */
Scape.prototype.getImages = function (bucketID, opts = {}) {
    var page = ''
    if (opts.page) page = ('page/').concat(opts.page)

    return new Promise((resolve, reject) => {
        axios.get(`${this.hostname}/buckets/${bucketID}/images/${page}`, {
            headers: this.getHeaders(),
        })
        .then(res => resolve(res.data))
        .catch(err => reject(err))
    })
}

/**
 * GET /images/{uuid}
 *
 * Get single image
 * 
 * @returns {Promise} 
 */
Scape.prototype.getImage = function (imageUUID) {
    return new Promise((resolve, reject) => {
        axios.get(`${this.hostname}/image/${imageUUID}`, {
            headers: this.getHeaders(),
        })
        .then(res => resolve(res.data))
        .catch(err => reject(err))
    })
}


// Upload

/**
 * .upload()
 *
 * @param {Object} opts Object of config options
 *   @param {String} [opts.hostname] hostname
 *   @param {String} [opts.hostnameUpload] hostnameUpload
 *   @param {String} [opts.token]
 * @returns {Promise} 
 */
Scape.prototype.upload = function (file, opts, edit) {
    // var token = this.apiToken;
    // if (typeof token !== 'string') token = this.getEphemeralToken();

    return new Promise((resolve, reject) => {
        axios.post(this.hostnameUpload, {
            image: file,
            options: opts,
            edit: edit,
        }, {
            headers: this.getHeaders(),
        })
        .then(res => resolve(res.data))
        .catch(err => reject(err))
    })
}


// Misc

/**
 * GET /group/stats
 *
 * Get group stats
 *
 * @returns {Promise} 
 */
Scape.prototype.getStats = function () {
    return new Promise((resolve, reject) => {
        axios.get(`${this.hostname}/group/stats`, {
            headers: this.getHeaders()
        })
        .then(res => resolve(res.data))
        .catch(err => reject(err))
    })
}

module.exports = Scape
